@@ -4,10 +4,6 @@
  * included with libxml++ as the file COPYING.
  */
 
-#include <libxml++config.h> // LIBXMLXX_DISABLE_DEPRECATED
-
-#ifndef LIBXMLXX_DISABLE_DEPRECATED
-
 #include <libxml++/schema.h>
 
 #include <libxml/tree.h>
@@ -25,7 +21,7 @@ Schema::Schema(_xmlSchema* schema)
 }
 
 Schema::Schema(Document* document, bool embed)
-: impl_(nullptr)
+: impl_(0)
 , embedded_doc_(false)
 {
   set_document(document, embed);
@@ -40,30 +36,18 @@ void Schema::set_document(Document* document, bool embed)
 {
   release_underlying();
 
-  bool has_created_document = false;
-  if (!document)
-  {
-    document = new Document();
-    has_created_document = true;
-  }
-
-  xmlResetLastError();
-  auto context = xmlSchemaNewDocParserCtxt( document->cobj() );
+  xmlSchemaParserCtxtPtr context = xmlSchemaNewDocParserCtxt( document->cobj() );
 
   if(!context)
   {
-    if (has_created_document)
-      delete document;
-    throw parse_error("Schema could not be parsed.\n" + format_xml_error());
+   throw parse_error("Schema could not be parsed");
   }
   
   impl_ = xmlSchemaParse(context);
   if(!impl_)
   {
     xmlSchemaFreeParserCtxt(context);
-    if (has_created_document)
-      delete document;
-    throw parse_error("Schema could not be parsed.\n" + format_xml_error());
+    throw parse_error("Schema could not be parsed");
   }
 
   impl_->_private = this;
@@ -73,22 +57,22 @@ void Schema::set_document(Document* document, bool embed)
 
 Glib::ustring Schema::get_name() const
 {
-  return (impl_ && impl_->name) ? (char*)impl_->name : "";
+  return (char*)impl_->name;
 }
 
 Glib::ustring Schema::get_target_namespace() const
 {
-  return (impl_ && impl_->targetNamespace) ? (char*)impl_->targetNamespace : "";
+  return (char*)impl_->targetNamespace;
 }
 
 Glib::ustring Schema::get_version() const
 {
-  return (impl_ && impl_->version) ? (char*)impl_->version : "";
+  return (char*)impl_->version;
 }
 
 void Schema::release_underlying()
 {
-  if(embedded_doc_ && impl_ && impl_->doc && impl_->doc->_private)
+  if(embedded_doc_ && impl_ && impl_->doc->_private)
   {
     delete (Document*) impl_->doc->_private;
     embedded_doc_ = false;
@@ -97,24 +81,24 @@ void Schema::release_underlying()
   if(impl_)
   {
     xmlSchemaFree(impl_);
-    impl_ = nullptr;
+    impl_ = 0;
   }
 }
 
 Document* Schema::get_document()
 {
-  if (!(impl_ && impl_->doc))
-    return nullptr;
-
-  if (!impl_->doc->_private) // Possible if *this was created with Schema(xmlSchema* schema).
-    new Document(impl_->doc); // Sets impl_->doc->_private
-
-  return (Document*) impl_->doc->_private;
+  if(impl_)
+    return (Document*) impl_->doc->_private;
+  else
+    return 0;
 }
 
-const Document* Schema::get_document() const
+const Document* Schema::get_document()const
 {
-  return const_cast<Schema*>(this)->get_document();
+  if(impl_)
+    return (Document*) impl_->doc->_private;
+  else
+    return 0;
 }
 
 _xmlSchema* Schema::cobj()
@@ -128,4 +112,3 @@ const _xmlSchema* Schema::cobj() const
 }
 
 } //namespace xmlpp
-#endif // LIBXMLXX_DISABLE_DEPRECATED
